
##' Setup of the wood consumption
##'
##' This function calculates the wood consumption of the village
##' @title wood_consumption
##' @param population The number of people living in the village; default: 3800;
##' @param forest_type The type of the forest; choose between "oak_mixed_forest" and "olive_mixed_forest"; default "oak_mixed_forest"
##' @param consumption_energy_person_a_GJ The consumption of energy per person in GJ; default 8 (FAO 2013)
##' @param consumption_fuelwood_person_a_kg The fuelwood consumption per person; default 551
##' @param productivity The productivity of the mentioned forest_type in t/ha. default: 12.9 (source average wood productivity in agroforestry; (FAO 2013))
##' @param calorific_value The calorific value of wood in MJ/kg; default: 18.62 (Olive wood (FAO 2013))
##' @param ... Further arguments passed to other functions
##' @return A tibble
##' @export
##'

setup_wood_consumption <- function(population,
                                   forest_type,
                                   ...) {


  data(wood_cons, package = "LandUseQuantifieR")

  data(wood_prod, package = "LandUseQuantifieR")

  wood_produc <- wood_prod %>%
  dplyr::filter(wood_prod$type %in% get("forest_type"))

  wood_productivity <- dplyr::data_frame(
    calorific_value = wood_produc$productivity_in_cbm_per_ha_per_a,
    productivity_tha = wood_produc$productivity_in_cbm_per_ha_per_a,
    productivity_kgm2 = (productivity_tha * 1000) / 10000,
    productivity_kgha = productivity_tha * 1000)


  wood <- dplyr::data_frame(
    productivity_tha = wood_produc$productivity_in_cbm_per_ha_per_a,
    calorific_value = wood_produc$productivity_in_cbm_per_ha_per_a,
    productivity_kgm2 = wood_productivity$productivity_kgm2,
    productivity_kgha = wood_productivity$productivity_kgha,
    consumption_energy_person_a_GJ = wood_cons$consumption_energy_person_a_GJ,
    consumption_energy_person_a_MJ = consumption_energy_person_a_GJ * 1000,
    consumption_wood_person_a = consumption_energy_person_a_MJ / wood_produc$calorific_value_in_MJ_per_kg,
    consumption_fuelwood_person_a_kg = wood_cons$consumption_fuelwood_person_a_kg,
    consumption_fuelwood_person_a_GJ = (wood_cons$consumption_fuelwood_person_a_kg * wood_produc$calorific_value_in_MJ_per_kg) / 1000,
    req_person_a = (consumption_wood_person_a + consumption_fuelwood_person_a_kg) / 2,
    req_village_a = req_person_a * population,
    area_wood_production_m2 = req_village_a / productivity_kgm2,
    area_wood_production_ha = req_village_a / productivity_kgha
    )

  return(wood)

}

##' Setup of the kcal etc. values for wine
##'
##' This function sets up the kcal etc. values for wine in form of a tibble
##' @title setup_wine_complex
##' @param population The number of people living in a village
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @param ... Further arguments passed to other functions
##' @return A tibble
##' @export
setup_wine_complex <- function(population,
                               food_df,
                               ...) {

  kcal_human_year <- setup_population_simple(population,
                                             ...)$KcalHumanYear

  data(plants, package = "LandUseQuantifieR")

  wine_prod <- plants %>%
  dplyr::filter(species == "wine")

  wine <- dplyr::data_frame(
    Productivity = wine_prod$productivity,
    Calories = wine_prod$calories,
    EnergyPerArea = Productivity * Calories,
    PropDiet = food_df$Proportion[food_df$Product=="wine"],
    kcal_person_a = PropDiet *  kcal_human_year,
    kcal_ha_net = EnergyPerArea,
    area_person = kcal_person_a / kcal_ha_net,
    area_village = area_person * population,
    yield = area_village * Productivity)

  return(wine)
}


##' Setup of the kcal etc. values for olive_oil
##'
##' This function sets up the kcal etc. values for olive_oil in form of a tibble
##' @title setup_olive_oil_complex
##' @param population The number of people living in a village
##' @param households The number of households; default: population / 5, where 5 is the number of people living in a household
##' @param ... Further arguments passed to other functions
##' @return A tibble
##' @export
setup_olive_oil_complex <- function(population,
                            households,
                            olive_oil_storage = .25,
                            rich_people_factor = .66,
                            ...) {

  kcal_human_year <- setup_population_simple(population,
                                             ...)$KcalHumanYear

  data(plants, package = "LandUseQuantifieR")

  olive_oil_production <- plants %>%
    dplyr::filter(species == "olive_oil")

  olive_oil <- dplyr::data_frame(
    Productivity = olive_oil_production$productivity,
    oil_bathing = 2.5,
    oil_lighting = (households * 50) / population,
    area_bathing = oil_bathing / (Productivity * (1 - olive_oil_storage)),
    area_lighting = oil_lighting / (Productivity * (1 - olive_oil_storage)),
    area_village = (population * olive_oil_storage) + (area_bathing + area_lighting) * population,
    yield = (area_village * rich_people_factor) * Productivity)

  return(olive_oil)
}


##' Setup of the ceramics production
##'
##' This function sets up the required wood for ceramic production
##' @title setup_ceramics_complex
##' @param population The number of people living in the village
##' @param vessels_person The number of vessels per person; default 20;
##' @param mean_weight_vessel_kg The mean weight of a vessel in kg; default 1
##' @param mean_weight_wine_amphora_kg The  mean weight of wine amphora in kg; default 10
##' @param mean_weight_oil_amphora_kg The mean weight of olive oil amphora in kg; default 10
##' @param ratio_wood_ceramics The ratio of wood to ceramics; default .907184; source: Kahn 2008
##' @param food_df The product and the proportion of the consumed food
##' @param forest_type The type of the forest; choose between "oak_mixed_forest" and "olive_mixed_forest"; default "oak_mixed_forest"
##' @param households the of households
##' @param ... Further arguments passed to other functions
##' @return A tibble
##' @export
setup_ceramics_complex <- function(population,
                                   forest_type,
                                   food_df,
                                   households,
                                   olive_oil_yield,
                           ...) {

  productivity_tha <- setup_wood_consumption(population,
                                             forest_type,
                                             ...
                                             )$productivity_tha

  productivity_kgha <- setup_wood_consumption(population,
                                              forest_type,
                                              ...
                                              )$productivity_tha * 1000

  wine_yield <- if ("wine" %in% food_df$Product)
                    {setup_wine_complex(population,
                                        food_df,
                                        ...)$yield}
                else {(0)}



  data(ceramics_production, package = "LandUseQuantifieR")

  ceramics <- dplyr::data_frame(
    vessels_person = ceramics_production$vessels_person, # Weiberg et al. (in prep.)
    vessels_village = ceramics_production$vessels_person * population,
    mean_weight_vessel_kg = ceramics_production$mean_weight_vessel_kg,
    mean_weight_wine_amphora_kg = ceramics_production$mean_weight_wine_amphora_kg,
    mean_weight_oil_amphora_kg = ceramics_production$mean_weight_oil_amphora_kg,
    ratio_wood_ceramics = ceramics_production$ratio_wood_ceramics,
    amphora_production_wine = ((wine_yield * .5) / 2) / 21.6,
    amphora_production_olive_oil = olive_oil_yield / 39.4,
    weight_ceramics = (vessels_village * mean_weight_vessel_kg) +
      (amphora_production_wine * mean_weight_wine_amphora_kg) +
      (amphora_production_olive_oil * mean_weight_oil_amphora_kg),
    wood_req_ceramics = weight_ceramics * ratio_wood_ceramics,
    area_woodlot_ceramics = wood_req_ceramics / productivity_kgha)

  return(ceramics)
}

##' Setup of the bronze production
##'
##' This function sets up the amount of wood required for bronze production
##' @title setup_bronze_complex
##' @param population The number of people living in the village
##' @param forest_type The type of the forest; choose between "oak_mixed_forest" and "olive_mixed_forest"; default "oak_mixed_forest"
##' @param energy_req_melt_copper The amount of energy required to melt copper; default 60000 KJ
##' @param energy_req_melt_tin The amount of energy required to melt tin; default 59000 KJ
##' @param energy_req_melt_metals The amount of energy required to melt metal; default 59000 KJ
##' @param weight_melted_bronze_person The amount of weighted metal per person (kg/capita); default 2
##' @param ... Further arguments passed to other functions
##' @return A tibble
##' @export
setup_bronze_complex <- function(population,
                         forest_type,
                         ...) {

  calorific_value <- setup_wood_consumption(population,
                                              forest_type,
                                              ...
                                            )$calorific_value
  productivity_tha <- setup_wood_consumption(population,
                                             forest_type,
                                             ...
                                             )$productivity_tha

  productivity_kgha <- setup_wood_consumption(population,
                                              forest_type,
                                              ...
                                              )$productivity_tha * 1000


  data(bronze_production, package = "LandUseQuantifieR")

  bronze <- dplyr::data_frame(
    energy_req_melt_copper = bronze_production$energy_req_melt_copper, #KJ
    energy_req_melt_tin = bronze_production$energy_req_melt_tin, # KJ
    weight_melted_bronze_person = bronze_production$weight_melted_bronze_person, # (90\% copper, 10\% tin)
    weight_melted_bronze_village = weight_melted_bronze_person * population,
    energy_req_melt_metals = bronze_production$energy_req_melt_metals,
    ratio_wood_metal = energy_req_melt_metals / (calorific_value * 1000),
    wood_req = weight_melted_bronze_village * ratio_wood_metal,
    area_woodlot = wood_req / productivity_kgha)


return(bronze)

}

##' Setup of the iron production
##'
##' This function sets up the amount of wood  required for iron production
##' @title setup_iron_complex
##' @param population The number of people living in the village
##' @param forest_type The type of the forest; choose between "oak_mixed_forest" and "olive_mixed_forest"; default "oak_mixed_forest"
##' @param iron_person_kg The amount of iron per person as numeric; default 2; source:  Weiberg et al. (in prep.)
##' @param ratio_charcoal_iron The ratio of charcoal to iron as numeric; default 82
##' @param ratio_charcoal_wood The ration of charcoal to wood as numeric; default 4
##' @param ... Further arguments passed to other functions
##' @return A tibble with the characteristics of Iron
##' @export
setup_iron_complex <- function(population,
                               forest_type,
                               ...) {

  calorific_value <- setup_wood_consumption(population,
                                            forest_type,
                                            ...
                                            )$calorific_value

  productivity_tha <- setup_wood_consumption(population,
                                             forest_type,
                                             ...
                                             )$productivity_tha

  productivity_kgha <- setup_wood_consumption(population,
                                              forest_type,
                                              ...
                                              )$productivity_tha * 1000


  data(iron_production, package = "LandUseQuantifieR")

  iron <- dplyr::data_frame(
    iron_person_kg = iron_production$iron_person_kg,
    ratio_charcoal_iron = iron_production$ratio_charcoal_iron,
    ratio_charcoal_wood = iron_production$ratio_charcoal_wood,
    req_person_iron_kg = prod(iron_person_kg, ratio_charcoal_iron, ratio_charcoal_wood),
    req_person_iron_GJ = (req_person_iron_kg * calorific_value) / 1000,
    req_village_iron_kg = req_person_iron_kg * population,
    area_woodlot_iron = req_village_iron_kg / productivity_kgha)
  return(iron)
}


##' Setup the kcal etc. values for flax
##'
##' This function sets up the kcal etc. values for flax in form of a tibble
##' @title setup_flax_complex
##' @param population The number of people living in the village
##' @param productivity The productivity of the flax in kg/ha;  default 804
##' @param Linen_need_kg The amount of linen needed (kg); default .1
##' @param Flax_kg_in_100g_linen The amount of flax (kg) per 100g linen; default 1
##' @param PropPop The proportion of the population that uses flax; defaul .65
##' @param clothes_req Whether clothes are required; default NA
##' @return A tibble
##' @export
setup_flax_complex <- function(population) {
  flax <- dplyr::data_frame(
    Productivity = flax_production$productivity,
    Linen_need_kg = flax_production$Linen_need_kg,
    Flax_kg_in_100g_linen = flax_production$Flax_kg_in_100g_linen,
    PropPop = flax_production$PropPop,
    Linen_req_kg = population * PropPop * Linen_need_kg,
    flax_req_kg = population * PropPop * Flax_kg_in_100g_linen,
    area_req = flax_req_kg / Productivity)

  return(flax)
}

