##' Setup the crop and legume diet characteristics
##'
##' This function creates a data.frame of the diet characteristics of a village that is used in subsequent calculations;
##' @title setup_diet_crops_medium
##' @param population The number of people living in a village
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @param kcal_req_human_year The per capita kcal requirement of the year; if nothing is supplied, it is calculated based on setup_population function
##' @param loss The loss due to storage, waste loss or saving for next years planting; default .5
##' @return A list with a numeric (required hectar to grow plants to sustain the population) and a data.frame that supplies the individual values
##' @export
setup_diet_crops_medium <- function(population,
                                    food_df,
                                    kcal_req_human_year,
                                    loss = .5) {

  proportion_df <- crop_selector(food_df)

  diet <- proportion_df %>%
    dplyr::mutate(
      Kcal_requirements_percapita = c(
        Proportion * kcal_req_human_year
      ),
      area_req_percapita = c(
        Kcal_requirements_percapita / ((productivity * calories) * (1 - loss))
      ),
      area_req_village = c(
        area_req_percapita * population
      ),
      yield = c(area_req_village * productivity
      ),
      yield_unit = yield_units
      )

  return(list(AreaCrops = sum(diet$area_req_village),
              diet = diet))
}

##' Setup the general arboriculture and gathered plants diet characteristics
##'
##' This function creates a data.frame of the diet characteristics of a village that is used in subsequent calculations;
##' @title setup_diet_arbori_gather_medium
##' @param population The number of people living in a village
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @param kcal_req_human_year The per capita kcal requirement of the year; if nothing is supplied, it is calculated based on setup_population function
##' @param loss The loss due to storage, waste loss or saving for next years planting; default .5
##' @return A list with a numeric (required hectar to grow plants to sustain the population) and a data.frame that supplies the individual values
##' @export
setup_diet_arbori_gather_medium <- function(
  population,
  food_df,
  kcal_req_human_year,
  loss = .5) {

  proportion_df <- arbori_selector(food_df)

  diet <- proportion_df %>%
    dplyr::mutate(
      Kcal_requirements_percapita = c(
        Proportion * kcal_req_human_year
      ),
      area_req_percapita = c(
        Kcal_requirements_percapita / ((productivity * calories) * (1 - loss))
      ),
      area_req_village = c(
        area_req_percapita * population
      ),
      yield = c(area_req_village * productivity
      ),
      yield_unit = yield_units
      )

  return(list(AreaArboriGather = sum(diet$area_req_village),
              diet = diet))
}



##' Setup the general meat diet characteristics
##'
##' This function creates a data.frame of the diet characteristics of a village that is used in subsequent calculations;
##' @title setup_diet_pasture_medium
##' @param population The number of people living in a village
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @param kcal_req_human_year The per capita kcal requirement of the year; if nothing is supplied, it is calculated based on setup_population function
##' @return A list with a numeric (required hectar for pasture to sustain the animal herd) and a data.frame that supplies the individual values
##' @export
setup_diet_pasture_medium <- function(population,
                                      food_df,
                                      kcal_req_human_year
                                      ) {

  kcal_req_village_year <- kcal_req_human_year * population

  diet <- pasture_selector(food_df) %>%
    nd_herd_medium(input_animals = .,
                   kcal_req_village_year = kcal_req_village_year,
                   proportion = .$Proportion)

  return(list(AreaPasture = sum(diet$area_req_village),
              diet = diet))
}


##' Setup the general game diet characteristics
##'
##' This function creates a data.frame of the diet characteristics of a village that is used in subsequent calculations. To generate more realistic results, the number of required animals is rounded up to the nearest whole number;
##' @title setup_diet_game_medium
##' @param population The number of people living in a village
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @param kcal_req_human_year The per capita kcal requirement of the year; if nothing is supplied, it is calculated based on setup_population function
##' @return A list with a numeric (required hectar to grow plants to sustain the population) and a data.frame that supplies the individual values
##' @export
setup_diet_game_medium <-
  function(population,
           food_df,
           kcal_req_human_year) {

  proportion_df <- game_selector(food_df)

  diet <- proportion_df %>%
    dplyr::mutate(
      Kcal_requirements_percapita = c(
        Proportion * kcal_req_human_year
      ),
      kcal_req_village_year = population * Kcal_requirements_percapita,
      meat_yield_kg = meat_yield,
      meat_yield_kcal_slaughtered = meat_yield_kg * calories,
      calories_slaughter =
        kcal_req_village_year * (Proportion * (1/nrow(proportion_df))),
      number_req_for_calories =
        ceiling(calories_slaughter / meat_yield_kcal_slaughtered)
      )

  return(list(diet = diet))
}





##' Calculate herd characteristics when nothing specific is known
##'
##' This is a small helper function that is called when there are no specific information or empirical evidences to calculate the village meat requirements. To generate more realistic results, the number of required animals is rounded up to the nearest whole number. The function imports the data on animals and animal_products and takes the mean of those values; besides, fodder-plants are imported and all those plants that are also crops are removed from the dataset
##' @title nd_herd_medium
##' @param input_animals The animals input of the data.frame
##' @param kcal_req_village_year The kcal requirements of the village per year
##' @param proportion The proportion of the consumed foodtype in \%
##' @return A data.frame
nd_herd_medium <- function(input_animals, kcal_req_village_year, proportion) {
  data(animals, package = "LandUseQuantifieR")
  data(animal_products, package = "LandUseQuantifieR")
  data(plants, package = "LandUseQuantifieR")

  fodder <- plants %>%
    dplyr::filter(type=="fodder_crops") %>%
    dplyr::filter(species != "barley",
                  species != "emmer",
                  species != "einkorn",
                  species != "millet",
                  species != "lentil")

  selected_animals <- animals %>%
    dplyr::filter(is.element(animal, input_animals$Product)) %>%
    dplyr::left_join(y = input_animals,
                     by = c("animal" = "Product")) %>%
    dplyr::group_by(animal)

  nd_mixed_animal <- dplyr::data_frame(
    animal = selected_animals$animal,
    type = selected_animals$type.x,
    weight = selected_animals$weight,
    Prop_herd = selected_animals$herd_composition,
    calory_requirements = selected_animals$calory_requirements,
    meat_yield_factor = selected_animals$meat_yield_factor,
    meat_yield = selected_animals$meat_yield,
    Slaughter = selected_animals$slaughter_factor,
    slaughter_factor = .3,
    meat_yield_kg = weight * meat_yield_factor,
    meat_yield_kcal_slaughtered = meat_yield_kg * selected_animals$calories,
    calories_slaughter =
      kcal_req_village_year * selected_animals$Proportion * Slaughter,
    number_req_for_calories =
      ceiling(calories_slaughter / meat_yield_kcal_slaughtered),
    number_req_supply_meat =
      ceiling(number_req_for_calories / slaughter_factor * Prop_herd),
    total_herd =
      number_req_supply_meat + number_req_for_calories,
    Kcal_animal_a =
      (calory_requirements * 365) * total_herd,
    area_req_village =
      Kcal_animal_a / (mean(fodder$calories) * mean(fodder$productivity))
  )
  return(nd_mixed_animal)
}


##' Convert a data frame to a lower case
##'
##' This is a small helper function that is called data_checker and it converts the data given in the food_df into lower case
##' @title data_checker
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @return Parameter food_df

data_checker <- function(food_df) {
  food_df %<>%
    lapply(tolower) %>%
    dplyr::as_data_frame() %>%
    dplyr::mutate(Proportion = as.numeric(Proportion))
  return(food_df)
}

##' Combining, filtering and selecting data sets
##'
##' This small function called abori_selector adds new variables to one dataset from another dataset, filters the data input and selects the references for the users data input from the packages data
##' @title crop_selector
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @return Parameter food_df

crop_selector <- function(food_df) {
  data(plants, package = "LandUseQuantifieR")
  food_df %<>%
    data_checker %>%
    dplyr::left_join(
      x = .,
      y = plants,
      by = c("Product" = "species")) %>%
    dplyr::filter(grepl(pattern = "cereals|legumes",
                        x = type)) %>%
    dplyr::select(-reference_calories, -reference_productivity)

  return(food_df)
}

##' Combining, filtering and selecting data sets
##'
##' This small function called abori_selector adds new variables to one dataset from another dataset, filters the data input and selects the references for the users data input from the packages data
##' @title abori_selector
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @return Parameter food_df

arbori_selector <- function(food_df) {
  data(plants, package = "LandUseQuantifieR")
  food_df %<>%
    data_checker %>%
    dplyr::left_join(
      x = .,
      y = plants,
      by = c("Product" = "species")) %>%
    dplyr::filter(type == "arboriculture/gathered") %>%
    dplyr::select(-reference_calories, -reference_productivity)

  return(food_df)
}

##' checking the spelling of the animal data input
##'
##' This is a small helper function that is called spelling_checker and it allows the user to use plural or singular for data input
##' @title spelling_checker
##' @param animal_name The name of the animal
##' @return Parameter animal_name
spelling_checker <- function(animal_name) {
  animal_name <- ifelse(test = animal_name == "pig",
                         yes = "pigs",
                         no = animal_name)
  animal_name <- ifelse(test = animal_name == "goat",
                         yes = "goats",
                         no = animal_name)
  animal_name <- ifelse(test = animal_name == "cattles",
                         yes = "cattle",
                         no = animal_name)
  return(animal_name)
  }

##' Combining, filtering, selecting and renaming data sets
##'
##' This small function called abori_selector adds new variables to one dataset from another dataset, filters the data input and selects the references for the users data input from the packages data
##' @title pasture_selector
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @return Parameter food_df

pasture_selector <- function(food_df) {
  data(animal_products, package = "LandUseQuantifieR")

  food_df %<>%
    data_checker %>%
    dplyr::mutate(Product = spelling_checker(Product)) %>%
    dplyr::left_join(
      x = .,
      y = animal_products,
      by = c("Product" = "animal")) %>%
    dplyr::filter(product == "meat") %>%
    dplyr::select(-reference_calories) %>%
    dplyr::rename(type = product)
  return(food_df)
}

##' Combining and selecting data sets
##'
##' This small function called abori_selector adds new variables to one dataset from another dataset and selects the references for the users data input from the packages data
##' @title game_selector
##' @param food_df The consumed food and the proportion in \% of the consumed food
##' @return Parameter food_df

game_selector <- function(food_df) {
  data(game, package = "LandUseQuantifieR")
  food_df %<>%
    data_checker %>%
    dplyr::left_join(
      x = .,
      y = game,
      by = c("Product" = "animal")) %>%
    dplyr::select(-references_calories, -references_weight) %>%
    na.omit

  return(food_df)
}
