##' Calculate the energy requirements of a village
##'
##' Calculate the energy requirements of a village based on a given
##' population size
##'
##' This functions is a wrapper for more comprehensive functions calculating the nutritional (kcal) requirements of a village; These are not implemented yet; Note that more comprehensive approaches, e.g. considering a death-table of a village, require a lot more data that is often not available
##'
##' @title setup_population_simple
##' @param population The number of people living in a village
##' @param mode The calculation mode; currently only "simplified" (the default) and "medium" is implemented; further modes will be added soon and require further input data
##' simplified approach takes the mean of empirical values of daily kcal requirements for active females and males in the age between 10 and 50 (source for daily requirements:  \url{http://www.cnpp.usda.gov/sites/default/files/usda_food_patterns/EstimatedCalorieNeedsPerDayTable.pdf})
##' @param kcal_requirement optional; default NULL
##' @return A list with the elements: VillagePeople, KcalHuman, KcalHumanYear, KcalVillageYear
##' @export
setup_population_simple <- function(population,
                             mode = "simplified",
                             kcal_requirement = NULL
                             ) {

  if (mode == "simplified") {
    data(kcal_req_age, package = "LandUseQuantifieR")

    kcal_human <- kcal_req_age %>%
      dplyr::filter(as.numeric(age) > 10,
                    as.numeric(age) < 50) %>%
      dplyr::select(male_active, female_active) %>%
      sapply(FUN = mean) %>%
      mean(.)
  }

  if (mode == "manual") {
    if(!is.null(kcal_requirement)) {
      kcal_human <- kcal_requirement
    } else {
      stop("kcal_requirement has to be given as single value.")
    }
  }

  kcal_human_year <- kcal_human * 365
  kcal_village_year <- kcal_human_year * population

  return(list(
    "Population" = population,
    "KcalHuman" = kcal_human,
    "KcalHumanYear" = kcal_human_year,
    "KcalVillageYear" = kcal_village_year))

}
