##' Setup the general vegetarian diet characteristics
##'
##' This function creates a data.frame of the diet characteristics of a village that is used in subsequent calculations;
##' @title setup_diet_veg
##' @param population The number of people living in a village; default 3800
##' @param consumed_plants The kind of consumed plants
##' @param proportion The proportion of diet in numbers [only used when propdiet is "nd_mixed"]; default is c(.5, .4) assuming that the diet of the population is a mix of crops, fruits, and meat (calculated using setup_diet_meat)
##' @param kcal_req_human_year The kcal a human requires per year; default NULL
##' @param loss The loss due to storage, waste loss or saving for next years planting; default: .5
##' @return A list with a numeric (required hectar to grow plants to sustain the population) and a data_frame that supplies the individual values
##' @export
setup_diet_veg <- function(population,
                           consumed_plants = NULL,
                           proportion = NULL,
                           kcal_req_human_year = NULL,
                           loss = .5) {

  if (is.null(consumed_plants)) {
    data(plants, package = "LandUseQuantifieR")
    proportion <- c(.5,.4)

    diet <- dplyr::data_frame(
      Catgory = c("crops","fruits"),
      Type = "undefined",
      Proportion = proportion,
      Productivity = c(
        mean(plants$productivity[is.element(plants$type, c("cereals", "legumes"))]),
        mean(plants$productivity[is.element(plants$type, c("arboriculture/gathered"))])
        ),
      Calories = c(
        mean(plants$calories[is.element(plants$type, c("cereals", "legumes"))]),
        mean(plants$calories[is.element(plants$type, c("arboriculture/gathered"))])
      ))
  }

  if (!is.null(consumed_plants)) {
    data(plants, package = "LandUseQuantifieR")

    proportion_df <- dplyr::data_frame(Species = consumed_plants,
                                Proportion = proportion)

    diet <- plants %>%
      dplyr::as.tbl() %>%
      dplyr::filter(species %in% consumed_plants) %>%
      dplyr::select(species, productivity, calories) %>%
      dplyr::rename(Species = species,
                    Productivity = productivity,
                    Calories = calories) %>%
      dplyr::left_join(y = proportion_df,
                       by = "Species")

    }

  diet <- diet %>%
    dplyr::mutate(
      Kcal_requirements_percapita = c(
        Proportion * kcal_req_human_year
      ),
      area_req_percapita = c(
        Kcal_requirements_percapita / ((Productivity * Calories) * (1 - loss))
      ),
      area_req_village = c(
        area_req_percapita * population
      ))

  return(list(AreaPlants = sum(diet$area_req_village),
              diet))
}



##' Setup the general meat diet characteristics
##'
##' This function creates a data.frame of the diet characteristics of a village that is used in subsequent calculations;
##' @title setup_diet_meat
##' @param population The number of people living in a village; default 3800
##' @param propdiet The proportions of the consumed food; currently only "nd_mixed" is implemented what is representative for no data and a mixed diet that consists of crops and fruits in equal amount (1/2); it takes the values supplied in the datasets "plants" and uses the mean values to return productivity (kcal/ha) and calory values
##' @param proportion The proportion of meat of the population's diet; default: .1
##' @param kcal_req The per capita kcal requirement; if is nothing is supplied, it is calculated based on setup_population function; default: "simplified"
##' @return A list with a numeric (required hectar for pasture to sustain the animal herd) and a data_frame that supplies the individual values
##' @export
setup_diet_meat <- function(population = 3800,
                            propdiet = "nd_mixed",
                            proportion = .1,
                            kcal_req = "simplified"
                            ) {

  if (kcal_req == "simplified") {
    kcal_req_village_year <- setup_population_simple(population = population)$KcalVillageYear
    }

  if (propdiet == "nd_mixed") {
    data(animal_products, package = "LandUseQuantifieR")

    herd <- nd_herd_simple(kcal_req_village_year = kcal_req_village_year, proportion = proportion)

    diet <- dplyr::data_frame(
      Catgory = c("meat"),
      Type = "undefined",
      Proportion = proportion,
      Calories = c(
        mean(animal_products$calories[is.element(animal_products$product, "meat")])
      )) %>%
      dplyr::bind_cols(herd)
  }
  return(list(AreaPasture = sum(diet$area_req_village),
              diet))
}

##' Calculate herd characteristics when nothing specific is known
##'
##' This is a small helper function that is called when there are no specific information or empirical evidences to calculate the village meat requirements. The function imports the data on animals and animal_products and takes the mean of those values; besides, fodder-plants are imported and all those plants that are also crops are removed from the dataset.
##' @title nd_herd_simple
##' @param kcal_req_village_year The kcal requirements of the village per year
##' @param proportion The proportion of the diet
##' @return A data_frame
nd_herd_simple <- function(kcal_req_village_year, proportion) {
  data(animals, package = "LandUseQuantifieR")
  data(animal_products, package = "LandUseQuantifieR")
  data(plants, package = "LandUseQuantifieR")

  fodder <- plants %>%
    dplyr::filter(type=="fodder_crops") %>%
    dplyr::filter(species != "barley",
                  species != "emmer",
                  species != "einkorn",
                  species != "millet",
                  species != "lentil")

  calories <- mean(animal_products$calories[is.element(animal_products$product, "meat")])

  nd_mixed_animal <- dplyr::data_frame(
      weight = mean(animals$weight),
      Prop_herd = 1,
      calory_requirements = mean(animals$calory_requirements),
      meat_yield_factor = mean(animals$meat_yield_factor),
      meat_yield = mean(animals$meat_yield),
      Slaughter = .2,
      slaughter_factor = .3, # i.e. the percentage of a herd that can be slaughtered
      meat_yield_kg = weight * meat_yield_factor,
      meat_yield_kcal_slaughtered = meat_yield_kg * calories,
      calories_slaughter = kcal_req_village_year * proportion * Slaughter,
      number_req_for_calories = calories_slaughter / meat_yield_kcal_slaughtered,
      number_req_supply_meat = number_req_for_calories / slaughter_factor * Prop_herd,
      total_herd = number_req_supply_meat + number_req_for_calories,
      Kcal_animal_a= (calory_requirements * 365) * total_herd,
      area_req_village = Kcal_animal_a / (mean(fodder$calories) * mean(fodder$productivity))
  )
  return(nd_mixed_animal)
}
